/************************************************************************/
/*				                        		*/
/*	LineFollowing_MRK                    		                */
/*						                  	*/
/*					                        	*/
/************************************************************************/
/*	Author: Jordan Retz						*/
/*	Copyright 2012, Digilent Inc.					*/
/************************************************************************/
/*  Module Description: 			             		*/
/*					                        	*/
/* This module implements a simple demo application that shows how to   */
/* program the RDK Line - Motor.                                        */
/*									*/
/*	Functionality:							*/
/*									*/
/* The application uses the PmodLS1, four IR proximity sensors, and two */
/* servo motors. Based on the outputs of the four sensors attached to   */
/* the Pmod, the robot will alter it's direction to stay on a line.     */
/* The robot will work best if the line is black electric tape on a     */
/* white surface.                                                       */
/*                                                                      */
/* The sensors are referred to as Far Right (FRS), Mid Right (MRS), Mid */
/* Left (MLS), and Far Left (FLS). These should be connected to S1, S2, */
/* S3, S4, respectively, on the PmodLS1. Left and right are determined  */
/* looking down on robot with sensor  bracket forward.                  */
/*                                                                      */
/*                                                                      */
/*      Instruction     FRS MRS MLS FLS    Function                     */
/*          0            0   0   0   0     Stop                         */
/*          1            0   0   0   1     Turn Right                   */
/*          2            0   0   1   0     Continue                     */
/*          3            0   0   1   1     Turn Right                   */
/*          4            0   1   0   0     Continue                     */
/*          5            0   1   0   1     Turn Right                   */
/*          6            0   1   1   0     Continue                     */
/*          7            0   1   1   1     Turn Right                   */
/*          8            1   0   0   0     Turn Left                    */
/*          9            1   0   0   1     Continue                     */
/*          10           1   0   1   0     Turn Left                    */
/*          11           1   0   1   1     Turn Right                   */
/*          12           1   1   0   0     Turn Left                    */
/*          13           1   1   0   1     Turn Left                    */
/*          14           1   1   1   0     Turn Left                    */
/*          15           1   1   1   1     Move Foward                  */
/*                                                                      */
/* When the BTN2 is pressed, the robot will begin moving. If BTN1 is    */
/* pressed, or, all sensors are off, the robot will stop.               */
/*                                                                      */
/* A PmodHB5 must be connected to the top row of header JD for the right*/
/* motor, and a PmodHB5 must be connected to the bottom row of header   */
/* JD for the left motor. The PmodLS1 must be connected to the top row  */
/* of header JJ.                                                        */
/*                                                                      */
/*     Programming:                                                     */
/*                                                                      */
/* Make sure that the MX4cK is plugged into the computer via USB cable, */
/* and that the board is powered.                                       */
/*                                                                      */
/* From the toolbar above, select "Tools->Board->MX4cK". From the Start */
/* menu, navitgate to "Start->Control Panel->Hardware and Sound->Device */
/* Manager". Select "Ports", and take note of what COM port (COMx) is   */
/* labeled "USB Serial Port". If you are unsure, unplug the MX4cK from  */
/* the computer and watch which COM port disappears, and then           */
/* reappears, when you plug the MX4cK back into the computer.           */
/* From the toolbar above, select "Tools->Serial Port" and then the COM */
/* port that you just found. Apply power to the MX4cK and click on the  */
/* "Upload" button above.                                               */
/*                                                                      */
/************************************************************************/
/*  Revision History:			        			*/
/*					                        	*/
/*	6/22/2012 (JordanR): created		       			*/
/*						                     	*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*                Include File Definitions                      */
/* ------------------------------------------------------------ */
#include <plib.h>

/* ------------------------------------------------------------ */
/*				Local Type Definitions		*/
/* ------------------------------------------------------------ */
/* These definitions apply if the Right Motor is connected to a PmodHB5 
** on the top row of JD, and the Left Motor is connected to a PmodHB5
** on the bottom row of JD.
*/
#define    pinLeftEn    29
#define    pinLeftDir   28
#define    pinRightEn   25
#define    pinRightDir  24

/* These definitions apply for MX4cK Rev B
*/
#define    pinBTN1      42
#define    pinBTN2      43

/* These definitions apply if the PmodLS1 is connected to the top row
** of JJ
*/
#define    pinFarRight  56
#define    pinMidRight  57
#define    pinMidLeft   58
#define    pinFarLeft   59

/* ------------------------------------------------------------ */
/*                Global Variables                              */
/* ------------------------------------------------------------ */
byte       bInstruction;
byte       bPrevInstruction;
byte       bRDKEnable;
byte       bBtn1;
byte       bBtn2;

/* ------------------------------------------------------------ */
/*                Forward Declarations                          */
/* ------------------------------------------------------------ */
void CheckSensors();
void CheckButtons();
void StopRobot();
void Forward();
void TurnRight();
void TurnLeft();


/* ------------------------------------------------------------ */
/*                Procedure Definitions                         */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/***  void setup()
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Initialize the system.
*/
void setup() {  
  /*Setup Input/Output on Pins
  */
  pinMode(pinRightDir, OUTPUT);
  pinMode(pinRightEn, OUTPUT);
  pinMode(pinLeftDir, OUTPUT);
  pinMode(pinLeftEn, OUTPUT);
  
  pinMode(pinBTN1, INPUT);
  pinMode(pinBTN2, INPUT);
  
  /* Disable Motors
  */
  digitalWrite(pinLeftEn, LOW);
  digitalWrite(pinRightEn, LOW);
  
  pinMode(pinFarRight, INPUT);
  pinMode(pinMidRight, INPUT);
  pinMode(pinMidLeft, INPUT);
  pinMode(pinFarLeft, INPUT);
  
  /* Misc.
  */
  StopRobot();
  bPrevInstruction = 0x00;

}

/* ------------------------------------------------------------ */
/***  void loop();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Main application loop.
*/
void loop() {
  CheckButtons();
  CheckSensors();
  
  /* Robot Stop if all Sensors are off or BTN1 pressed
  */
  if(bInstruction == 0 || bRDKEnable != 0x2) {
    
    StopRobot();
  }
  
  else if(bRDKEnable == 0x2 && bRDKEnable != bPrevInstruction) {
    
    if(bInstruction == 3 || bInstruction == 7 || bInstruction == 11 || bInstruction == 1 || bInstruction == 5){
      
      TurnRight();
    }
    else if(bInstruction == 12 || bInstruction == 13 || bInstruction == 14 || bInstruction == 8 || bInstruction == 10) {
      
      TurnLeft();
    }
    else {
      
      Forward();
    }
    
  }
  
  bPrevInstruction = bInstruction;

}

/* ------------------------------------------------------------ */
/***  void CheckSensors();
**
**  Parameters:
**    none
**
**  Return Value:
**  none
**
**  Errors:
**  none
**
**  Description:
**    Returns the state of the sensors. Must be connected to the 
**  top row of JA
*/
void CheckSensors() { 
    bInstruction = 0;
  
    byte bFRS = digitalRead(pinFarRight)*(0x8);
    byte bMRS = digitalRead(pinMidRight)*(0x4);
    byte bMLS = digitalRead(pinMidLeft)*(0x2);
    byte bFLS = digitalRead(pinFarLeft)*(0x1);
  
    bInstruction = (bFRS | bMRS | bMLS | bFLS);
}

/* ------------------------------------------------------------ */
/***  byte CheckButtons();
**
**  Parameters:
**    none
**
**  Return Value:
**  none
**
**  Errors:
**  none
**
**  Description:
**    Fetches the state of the buttons. 
*/
void CheckButtons() { 
    unsigned int i = 0;
    
    bBtn1 = 0;
    bBtn2 = 0;
  
    for(i = 0; i < 8; i++) {
      bBtn1 |= digitalRead(pinBTN1) << i;
      bBtn2 |= digitalRead(pinBTN2) << i;
      delay(1);
    }    
    
    if(bBtn2 & 0xFF) {
      bRDKEnable = 0x2;
    }
    else if(bBtn1 & 0xFF) {
      bRDKEnable = 0x1;
    }
    else {
      //continue previous instruction
    }
}

/* ------------------------------------------------------------ */
/***	void StopRobot();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Stops robot by finding Servo position and reducing or increasing to 90. 
**    Then detaches pins.
*/
void StopRobot() {
  
    /* Always disable the motors before changing their direction
    */
    OC2CONbits.ON = 0b0;
    OC3CONbits.ON = 0b0;
    T2CONbits.ON  = 0b0;
    
}

/* ------------------------------------------------------------ */
/***	void Forward();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Moves robot forward
*/
void Forward() {
   
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, LOW);
    digitalWrite(pinLeftDir, HIGH);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);

}

/* ------------------------------------------------------------ */
/***	void Backward();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Moves robot backward
*/
void Backward() {
  
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, HIGH);
    digitalWrite(pinLeftDir, LOW);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);

  
}

/* ------------------------------------------------------------ */
/***	void TurnRight();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Turns robot right
*/
void TurnRight() {  
  
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, HIGH);
    digitalWrite(pinLeftDir, HIGH);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);
    
}

/* ------------------------------------------------------------ */
/***	void TurnLeft();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**   Turns robot left
*/
void TurnLeft() {  
    
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, LOW);
    digitalWrite(pinLeftDir,LOW);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);
}
